@@ -1,6 +1,13 @@
 #ifndef EPIWORLD_DATABASE_MEAT_HPP
 #define EPIWORLD_DATABASE_MEAT_HPP
 
+#include <vector>
+#include <string>
+#include "config.hpp"
+#include "epiworld-macros.hpp"
+#include "misc.hpp"
+#include "database-bones.hpp"
+
 template<typename TSeq>
 inline void DataBase<TSeq>::reset()
 {
@@ -41,10 +48,12 @@ inline void DataBase<TSeq>::reset()
     hist_tool_counts.clear();    
 
     today_virus.resize(get_n_viruses());
-    std::fill(today_virus.begin(), today_virus.begin(), std::vector<int>(model->nstates, 0));
+    for (auto& virus_states : today_virus)
+        virus_states.assign(model->nstates, 0);
 
     today_tool.resize(get_n_tools());
-    std::fill(today_tool.begin(), today_tool.begin(), std::vector<int>(model->nstates, 0));
+    for (auto& tool_states : today_tool)
+        tool_states.assign(model->nstates, 0);
 
     hist_total_date.clear();
     hist_total_state.clear();
@@ -58,12 +67,15 @@ inline void DataBase<TSeq>::reset()
     transmission_target.clear();
     transmission_source_exposure_date.clear();
 
+    m_hospitalizations.reset();
+
     return;
 
 }
 
 template<typename TSeq>
 inline DataBase<TSeq>::DataBase(const DataBase<TSeq> & db) :
+    m_hospitalizations(db.m_hospitalizations),
     virus_id(db.virus_id),
     virus_name(db.virus_name),
     virus_sequence(db.virus_sequence),
@@ -190,6 +202,14 @@ inline void DataBase<TSeq>::record()
                 hist_virus_date.push_back(model->today());
                 hist_virus_id.push_back(p.second);
                 hist_virus_state.push_back(s);
+
+                #ifdef EPI_DEBUG
+                if (today_virus[p.second][s] < 0)
+                    throw std::logic_error(
+                        "The count of viruses in DataBase::record cannot be negative."
+                    );
+                #endif
+
                 hist_virus_counts.push_back(today_virus[p.second][s]);
 
             }
@@ -221,7 +241,7 @@ inline void DataBase<TSeq>::record()
             hist_total_counts.push_back(today_total[s]);
         }
 
-        for (auto cell : transition_matrix)
+        for (const auto& cell : transition_matrix)
             hist_transition_matrix.push_back(cell);
 
         // Now the diagonal must reflect the state
@@ -263,23 +283,44 @@ inline void DataBase<TSeq>::record_virus(Virus<TSeq> & v)
 {
 
     // If no sequence, then need to add one. This is regardless of the case
-    if (v.get_sequence() == nullptr)
-        v.set_sequence(default_sequence<TSeq>(
-            static_cast<int>(virus_name.size())
-            ));
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        if (v.get_sequence() == -1)
+            v.set_sequence(default_sequence<TSeq>(
+                static_cast<int>(virus_name.size())
+                ));
+    }
+    else
+    {
+        if (v.get_sequence() == nullptr)
+            v.set_sequence(default_sequence<TSeq>(
+                static_cast<int>(virus_name.size())
+                ));        
+    }
 
     // Negative id -> virus hasn't been recorded
     if (v.get_id() < 0)
     {
 
+        epiworld_fast_uint new_id = virus_id.size();
+        virus_name.push_back(v.get_name());
 
         // Generating the hash
-        std::vector< int > hash = seq_hasher(*v.get_sequence());
+        std::vector< int > hash;
+        EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+        {
+            hash = seq_hasher(v.get_sequence());
+            virus_id[hash] = new_id;
+            virus_sequence.push_back(v.get_sequence());
+        }
+        else
+        {
+            hash = seq_hasher(*v.get_sequence());
+            virus_id[hash] = new_id;
+            virus_sequence.push_back(*v.get_sequence());
+        }
+
 
-        epiworld_fast_uint new_id = virus_id.size();
-        virus_id[hash] = new_id;
-        virus_name.push_back(v.get_name());
-        virus_sequence.push_back(*v.get_sequence());
         virus_origin_date.push_back(model->today());
         
         virus_parent_id.push_back(v.get_id()); // Must be -99
@@ -293,11 +334,21 @@ inline void DataBase<TSeq>::record_virus(Virus<TSeq> & v)
 
         today_total_nviruses_active++;
 
-    } else { // In this case, the virus is already on record, need to make sure
+    }
+    else
+    { // In this case, the virus is already on record, need to make sure
              // The new sequence is new.
 
         // Updating registry
-        std::vector< int > hash = seq_hasher(*v.get_sequence());
+        std::vector< int > hash;
+        EPI_IF_TSEQ_LESS_EQ_INT(TSeq)
+        {
+            hash = seq_hasher(v.get_sequence());
+        }
+        else
+        {
+            hash = seq_hasher(*v.get_sequence());
+        }
         epiworld_fast_uint old_id = v.get_id();
         epiworld_fast_uint new_id;
 
@@ -308,7 +359,16 @@ inline void DataBase<TSeq>::record_virus(Virus<TSeq> & v)
             new_id = virus_id.size();
             virus_id[hash] = new_id;
             virus_name.push_back(v.get_name());
-            virus_sequence.push_back(*v.get_sequence());
+
+            EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+            {
+                virus_sequence.push_back(v.get_sequence());
+            }
+            else
+            {
+                virus_sequence.push_back(*v.get_sequence());
+            }
+
             virus_origin_date.push_back(model->today());
             
             virus_parent_id.push_back(old_id);
@@ -341,6 +401,11 @@ inline void DataBase<TSeq>::record_virus(Virus<TSeq> & v)
             today_virus[old_id][tmp_state]--;
             today_virus[new_id][tmp_state]++;
 
+            #ifdef EPI_DEBUG
+            if (today_virus[old_id][tmp_state] < 0)
+                throw std::logic_error("An entry in today_virus is negative.");
+            #endif
+
         }
 
     }
@@ -353,19 +418,41 @@ template<typename TSeq>
 inline void DataBase<TSeq>::record_tool(Tool<TSeq> & t)
 {
 
-    if (t.get_sequence() == nullptr)
-        t.set_sequence(default_sequence<TSeq>(
-            static_cast<int>(tool_name.size())
-        ));
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        if (t.get_sequence() == -1)
+            t.set_sequence(default_sequence<TSeq>(
+                static_cast<int>(tool_name.size())
+            ));
+    }
+    else
+    {
+        if (t.get_sequence() == nullptr)
+            t.set_sequence(default_sequence<TSeq>(
+                static_cast<int>(tool_name.size())
+            ));
+    }
 
     if (t.get_id() < 0) 
     {
 
-        std::vector< int > hash = seq_hasher(*t.get_sequence());
         epiworld_fast_uint new_id = tool_id.size();
-        tool_id[hash] = new_id;
         tool_name.push_back(t.get_name());
-        tool_sequence.push_back(*t.get_sequence());
+
+        std::vector< int > hash;
+        EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+        {
+            hash = seq_hasher(t.get_sequence());
+            tool_id[hash] = new_id;
+            tool_sequence.push_back(t.get_sequence());
+        }
+        else
+        {
+            hash = seq_hasher(*t.get_sequence());
+            tool_id[hash] = new_id;
+            tool_sequence.push_back(*t.get_sequence());
+
+        }
         tool_origin_date.push_back(model->today());
                 
         today_tool.push_back({});
@@ -378,7 +465,15 @@ inline void DataBase<TSeq>::record_tool(Tool<TSeq> & t)
     } else {
 
         // Updating registry
-        std::vector< int > hash = seq_hasher(*t.get_sequence());
+        std::vector< int > hash;
+        EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+        {
+            hash = seq_hasher(t.get_sequence());
+        }
+        else
+        {
+            hash = seq_hasher(*t.get_sequence());
+        }
         epiworld_fast_uint old_id = t.get_id();
         epiworld_fast_uint new_id;
         
@@ -388,7 +483,16 @@ inline void DataBase<TSeq>::record_tool(Tool<TSeq> & t)
             new_id = tool_id.size();
             tool_id[hash] = new_id;
             tool_name.push_back(t.get_name());
-            tool_sequence.push_back(*t.get_sequence());
+
+            EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+            {
+                tool_sequence.push_back(t.get_sequence());
+            }
+            else
+            {
+                tool_sequence.push_back(*t.get_sequence());
+            }
+            
             tool_origin_date.push_back(model->today());
                     
             today_tool.push_back({});
@@ -439,6 +543,25 @@ inline void DataBase<TSeq>::update_state(
         bool undo
 ) {
 
+    if (prev_state == new_state)
+        return; // No need to update if the state is the same
+
+    #ifdef EPI_DEBUG
+    // Checking ranges (should be within expected)
+    if ((prev_state >= model->nstates))
+        throw std::out_of_range(
+            "prev_state is out of range in DataBase::update_state"
+        );
+    if ((new_state >= model->nstates))
+        throw std::out_of_range(
+            "new_state is out of range in DataBase::update_state"
+        );
+    if (prev_state == new_state)
+        throw std::logic_error(
+            "prev_state and new_state are the same in DataBase::update_state"
+        );
+    #endif
+
     if (undo)
     {
 
@@ -461,11 +584,27 @@ template<typename TSeq>
 inline void DataBase<TSeq>::update_virus(
         epiworld_fast_uint virus_id,
         epiworld_fast_uint prev_state,
-        epiworld_fast_uint new_state
+        epiworld_fast_uint new_state,
+        bool undo
 ) {
 
-    today_virus[virus_id][prev_state]--;
-    today_virus[virus_id][new_state]++;
+    if (undo)
+    {
+        today_virus[virus_id][prev_state]++;
+        today_virus[virus_id][new_state]--;
+    }
+    else
+    {
+        today_virus[virus_id][prev_state]--;
+        today_virus[virus_id][new_state]++;
+    }
+
+    #ifdef EPI_DEBUG
+    if (today_virus[virus_id][prev_state] < 0)
+        throw std::logic_error("An entry in today_virus is negative.");
+    if (today_virus[virus_id][new_state] < 0)
+        throw std::logic_error("An entry in today_virus is negative (new_state).");
+    #endif
 
     return;
     
@@ -475,12 +614,20 @@ template<typename TSeq>
 inline void DataBase<TSeq>::update_tool(
         epiworld_fast_uint tool_id,
         epiworld_fast_uint prev_state,
-        epiworld_fast_uint new_state
+        epiworld_fast_uint new_state,
+        bool undo
 ) {
 
-
-    today_tool[tool_id][prev_state]--;    
-    today_tool[tool_id][new_state]++;
+    if (undo)
+    {
+        today_tool[tool_id][prev_state]++;
+        today_tool[tool_id][new_state]--;
+    }
+    else
+    {
+        today_tool[tool_id][prev_state]--;
+        today_tool[tool_id][new_state]++;
+    }
 
     return;
 
@@ -515,7 +662,7 @@ inline void DataBase<TSeq>::record_transition(
 
 template<typename TSeq>
 inline int DataBase<TSeq>::get_today_total(
-    std::string what
+    const std::string & what
 ) const
 {
 
@@ -550,19 +697,24 @@ inline void DataBase<TSeq>::get_today_virus(
     std::vector< int > & counts
     ) const
 {
-      
-    state.resize(today_virus.size(), "");
-    id.resize(today_virus.size(), 0);
-    counts.resize(today_virus.size(),0);
+    
+    // Total entries = number of viruses × number of states
+    size_t n_viruses = today_virus.size();
+    size_t n_states = model->states_labels.size();
+    size_t total_entries = n_viruses * n_states;
 
-    int n = 0u;
-    for (epiworld_fast_uint v = 0u; v < today_virus.size(); ++v)
-        for (epiworld_fast_uint s = 0u; s < model->states_labels.size(); ++s)
+    state.resize(total_entries, "");
+    id.resize(total_entries, 0);
+    counts.resize(total_entries, 0);
+
+    size_t n = 0u;
+    for (epiworld_fast_uint v = 0u; v < n_viruses; ++v)
+        for (epiworld_fast_uint s = 0u; s < n_states; ++s)
         {
             state[n]   = model->states_labels[s];
-            id[n]       = static_cast<int>(v);
-            counts[n++] = today_virus[v][s];
-
+            id[n]      = static_cast<int>(v);
+            counts[n]  = today_virus[v][s];
+            ++n;
         }
 
 }
@@ -601,8 +753,7 @@ inline void DataBase<TSeq>::get_hist_virus(
 ) const {
 
     date = hist_virus_date;
-    std::vector< std::string > labels;
-    labels = model->states_labels;
+    const auto& labels = model->states_labels;
     
     id = hist_virus_id;
     state.resize(hist_virus_state.size(), "");
@@ -625,8 +776,7 @@ inline void DataBase<TSeq>::get_hist_tool(
 ) const {
 
     date = hist_tool_date;
-    std::vector< std::string > labels;
-    labels = model->states_labels;
+    const auto& labels = model->states_labels;
     
     id = hist_tool_id;
     state.resize(hist_tool_state.size(), "");
@@ -639,6 +789,18 @@ inline void DataBase<TSeq>::get_hist_tool(
 
 }
 
+template<typename TSeq>
+inline void DataBase<TSeq>::get_today_transition_matrix(
+    std::vector< int > & counts
+) const
+{
+
+    counts = transition_matrix;
+
+    return;
+
+}
+
 template<typename TSeq>
 inline void DataBase<TSeq>::get_hist_transition_matrix(
     std::vector< std::string > & state_from,
@@ -702,6 +864,85 @@ inline void DataBase<TSeq>::get_hist_transition_matrix(
 
 }
 
+
+template<typename TSeq>
+inline void DataBase<TSeq>::get_active_cases(
+    std::vector<int> & date,
+    std::vector<int> & virus_id,
+    std::vector<int> & count
+) const 
+{
+
+    // Extracting useful sizes
+    size_t n_days    = model->get_ndays() + 1u; // Including day 0
+    size_t n_viruses = model->get_n_viruses();
+    
+    // Making room
+    date.assign(n_days * n_viruses, 0);
+    virus_id.assign(n_days * n_viruses, 0);
+    count.assign(n_days * n_viruses, 0);
+
+    for (size_t i = 0u; i < hist_virus_id.size(); ++i)
+    {
+
+        // With more viruses, there are more days
+        auto location = hist_virus_date[i] + hist_virus_id[i] * n_days;
+
+        date[location] = hist_virus_date[i];
+        virus_id[location] = hist_virus_id[i];
+        count[location] += hist_virus_counts[i];
+
+    }
+
+    return;
+    
+}
+
+template<typename TSeq>
+inline void DataBase<TSeq>::get_outbreak_size(
+    std::vector<int> & date,
+    std::vector<int> & virus_id,
+    std::vector<int> & outbreak_size
+) const 
+{
+
+    // Extracting useful sizes
+    size_t n_days    = model->get_ndays() + 1u; // Including day 0
+    size_t n_viruses = model->get_n_viruses();
+    
+    // Making room
+    date.assign(n_days * n_viruses, 0);
+    std::iota(date.begin(), date.end(), 0);
+
+    virus_id.assign(n_days * n_viruses, 0);
+    for (size_t v = 0u; v < n_viruses; ++v)
+        for (size_t d = 0u; d < n_days; ++d)
+            virus_id[d + v * n_days] = static_cast<int>(v);
+
+    outbreak_size.assign(n_days * n_viruses, 0);
+
+    // With more viruses, there are more days;
+    for (size_t i = 0u; i < transmission_date.size(); ++i)
+    {
+        outbreak_size[
+            transmission_date[i] + transmission_virus[i] * n_days
+        ] += 1;
+    }
+
+    // Now, we generate the cumulative sum
+    for (size_t v = 0u; v < n_viruses; ++v)
+    {
+        for (size_t d = 1u; d < n_days; ++d)
+        {
+            auto location = d + v * n_days;
+            outbreak_size[location] += outbreak_size[location - 1u];
+        }
+    }
+
+    return;
+    
+}
+
 template<typename TSeq>
 inline void DataBase<TSeq>::get_transmissions(
     std::vector<int> & date,
@@ -765,7 +1006,10 @@ inline void DataBase<TSeq>::write_data(
     std::string fn_transmission,
     std::string fn_transition,
     std::string fn_reproductive_number,
-    std::string fn_generation_time
+    std::string fn_generation_time,
+    std::string fn_active_cases,
+    std::string fn_outbreak_size,
+    std::string fn_hospitalizations
 ) const
 {
 
@@ -785,7 +1029,7 @@ inline void DataBase<TSeq>::write_data(
 
         file_virus_info <<
         #ifdef EPI_DEBUG
-            "thread" << "virus_id " << "virus " << "virus_sequence " << "date_recorded " << "parent\n";
+            "thread " << "virus_id " << "virus " << "virus_sequence " << "date_recorded " << "parent\n";
         #else
             "virus_id " << "virus " << "virus_sequence " << "date_recorded " << "parent\n";
         #endif
@@ -833,8 +1077,8 @@ inline void DataBase<TSeq>::write_data(
                 #endif
                 hist_virus_date[i] << " " <<
                 hist_virus_id[i] << " \"" <<
-                virus_name[hist_virus_id[i]] << "\" " <<
-                model->states_labels[hist_virus_state[i]] << " " <<
+                virus_name[hist_virus_id[i]] << "\" \"" <<
+                model->states_labels[hist_virus_state[i]] << "\" " <<
                 hist_virus_counts[i] << "\n";
     }
 
@@ -897,8 +1141,8 @@ inline void DataBase<TSeq>::write_data(
                 EPI_GET_THREAD_ID() << " " <<
                 #endif
                 hist_tool_date[i] << " " <<
-                hist_tool_id[i] << " " <<
-                model->states_labels[hist_tool_state[i]] << " " <<
+                hist_tool_id[i] << " \"" <<
+                model->states_labels[hist_tool_state[i]] << "\" " <<
                 hist_tool_counts[i] << "\n";
     }
 
@@ -990,25 +1234,155 @@ inline void DataBase<TSeq>::write_data(
         {
 
             for (int from = 0u; from < ns; ++from)
+            {
                 for (int to = 0u; to < ns; ++to)
+                {
+                    // Skipping the zeros
+                    auto counts = hist_transition_matrix[
+                        i * (ns * ns) + to * ns + from
+                    ];
+
+                    if (counts == 0)
+                        continue;
+
                     file_transition <<
                         #ifdef EPI_DEBUG
                         EPI_GET_THREAD_ID() << " " <<
                         #endif
-                        i << " " <<
-                        model->states_labels[from] << " " <<
-                        model->states_labels[to] << " " <<
-                        hist_transition_matrix[i * (ns * ns) + to * ns + from] << "\n";
+                        i << " \"" <<
+                        model->states_labels[from] << "\" \"" <<
+                        model->states_labels[to] << "\" " <<
+                        counts << "\n";
+                }
+            }
                 
         }
                 
     }
 
     if (fn_reproductive_number != "")
-        reproductive_number(fn_reproductive_number);
+        get_reproductive_number(fn_reproductive_number);
 
     if (fn_generation_time != "")
-        generation_time(fn_generation_time);
+        get_generation_time(fn_generation_time);
+
+    if (fn_active_cases != "")
+    {
+        std::vector< int > date;
+        std::vector< int > virus_id;
+        std::vector< int > count;
+        get_active_cases(date, virus_id, count);
+
+        std::ofstream file_active_cases(fn_active_cases, std::ios_base::out);
+        // Repeat the same error if the file doesn't exists
+        if (!file_active_cases)
+        {
+            throw std::runtime_error(
+                "Could not open file \"" + fn_active_cases +
+                "\" for writing.")
+                ;
+        }
+
+        file_active_cases <<
+            #ifdef EPI_DEBUG
+            "thread " << 
+            #endif
+            "date " << "virus_id virus " << "active_cases\n";
+
+        for (size_t i = 0u; i < date.size(); ++i)
+        {
+            if (count[i] > 0)
+                file_active_cases <<
+                    #ifdef EPI_DEBUG
+                    EPI_GET_THREAD_ID() << " " <<
+                    #endif
+                    date[i] << " " <<
+                    virus_id[i] << " \"" <<
+                    virus_name[virus_id[i]] << "\" " <<
+                    count[i] << "\n";
+        }
+
+    }
+
+    if (fn_outbreak_size != "")
+    {
+        std::vector< int > date;
+        std::vector< int > virus_id;
+        std::vector< int > outbreak_size;
+        get_outbreak_size(date, virus_id, outbreak_size);
+
+        std::ofstream file_outbreak_size(fn_outbreak_size, std::ios_base::out);
+        // Repeat the same error if the file doesn't exists
+        if (!file_outbreak_size)
+        {
+            throw std::runtime_error(
+                "Could not open file \"" + fn_outbreak_size +
+                "\" for writing.")
+                ;
+        }
+
+        file_outbreak_size <<
+            #ifdef EPI_DEBUG
+            "thread " << 
+            #endif
+            "date " << "virus_id virus " << "outbreak_size\n";
+
+        for (size_t i = 0u; i < date.size(); ++i)
+        {
+            if (outbreak_size[i] > 0)
+                file_outbreak_size <<
+                    #ifdef EPI_DEBUG
+                    EPI_GET_THREAD_ID() << " " <<
+                    #endif
+                    date[i] << " " <<
+                    virus_id[i] << " \"" <<
+                    virus_name[virus_id[i]] << "\" " <<
+                    outbreak_size[i] << "\n";
+        }
+
+    }
+
+    if (fn_hospitalizations != "")
+    {
+        std::vector< int > date;
+        std::vector< int > virus_id;
+        std::vector< int > tool_id;
+        std::vector< int > count;
+        std::vector< double > weight;
+        get_hospitalizations(date, virus_id, tool_id, count, weight);
+
+        std::ofstream file_hospitalizations(fn_hospitalizations, std::ios_base::out);
+        // Repeat the same error if the file doesn't exists
+        if (!file_hospitalizations)
+        {
+            throw std::runtime_error(
+                "Could not open file \"" + fn_hospitalizations +
+                "\" for writing.")
+                ;
+        }
+
+        file_hospitalizations <<
+            #ifdef EPI_DEBUG
+            "thread " << 
+            #endif
+            "date " << "virus_id " << "tool_id " << "count " << "weight\n";
+
+        for (size_t i = 0u; i < date.size(); ++i)
+        {
+            // Only write non-zero counts or weights
+            if (count[i] > 0 || weight[i] > 0.0)
+                file_hospitalizations <<
+                    #ifdef EPI_DEBUG
+                    EPI_GET_THREAD_ID() << " " <<
+                    #endif
+                    date[i] << " " <<
+                    virus_id[i] << " " <<
+                    tool_id[i] << " " <<
+                    count[i] << " " <<
+                    weight[i] << "\n";
+        }
+
+    }
 
 }
 
@@ -1078,7 +1452,7 @@ inline UserData<TSeq> & DataBase<TSeq>::get_user_data()
 }
 
 template<typename TSeq>
-inline MapVec_type<int,int> DataBase<TSeq>::reproductive_number()
+inline MapVec_type<int,int> DataBase<TSeq>::get_reproductive_number()
 const {
 
     // Checking size
@@ -1116,12 +1490,12 @@ const {
 }
 
 template<typename TSeq>
-inline void DataBase<TSeq>::reproductive_number(
+inline void DataBase<TSeq>::get_reproductive_number(
     std::string fn
 ) const {
 
 
-    auto map = reproductive_number();
+    auto map = get_reproductive_number();
 
     std::ofstream fn_file(fn, std::ios_base::out);
 
@@ -1157,60 +1531,54 @@ inline void DataBase<TSeq>::reproductive_number(
 }
 
 template<typename TSeq>
-inline std::vector< epiworld_double > DataBase<TSeq>::transition_probability(
-    bool print
+inline std::vector< epiworld_double > DataBase<TSeq>::get_transition_probability(
+    bool print,
+    bool normalize
 ) const {
 
-    auto states_labels = model->get_states();
+    const auto& states_labels = model->get_states();
     size_t n_state = states_labels.size();
     size_t n_days   = model->get_ndays();
     std::vector< epiworld_double > res(n_state * n_state, 0.0);
-    std::vector< epiworld_double > days_to_include(n_state, 0.0);
+    std::vector< epiworld_double > rowsums(n_state, 0.0);
 
-    for (size_t t = 1; t < n_days; ++t)
+    for (size_t t = 0; t < n_days; ++t)
     {
 
         for (size_t s_i = 0; s_i < n_state; ++s_i)
         {
-            epiworld_double daily_total = hist_total_counts[(t - 1) * n_state + s_i];
-
-            if (daily_total == 0)
-                continue;
-
-            days_to_include[s_i] += 1.0; 
 
             for (size_t s_j = 0u; s_j < n_state; ++s_j)
             {
-                #ifdef EPI_DEBUG
-                epiworld_double entry = hist_transition_matrix[
+                res[s_i + s_j * n_state] += (
+                    hist_transition_matrix[
+                        s_i + s_j * n_state +
+                        t * (n_state * n_state)
+                    ]
+                );
+                
+                rowsums[s_i] += hist_transition_matrix[
                     s_i + s_j * n_state +
                     t * (n_state * n_state)
-                    ];
-
-                if (entry > daily_total)
-                    throw std::logic_error(
-                        "The entry in hist_transition_matrix cannot have more elememnts than the total"
-                        );
-
-                res[s_i + s_j * n_state] += (entry / daily_total);
-                #else
-                    res[s_i + s_j * n_state] += (
-                        hist_transition_matrix[
-                            s_i + s_j * n_state +
-                            t * (n_state * n_state)
-                        ] / daily_total
-                    );
-                #endif
+                ];
+            
             }
 
         }
 
     }
 
-    for (size_t s_i = 0; s_i < n_state; ++s_i)
+    if (normalize)
     {
-        for (size_t s_j = 0; s_j < n_state; ++s_j)
-            res[s_i + s_j * n_state] /= days_to_include[s_i];
+        for (size_t s_i = 0; s_i < n_state; ++s_i)
+        {
+            // Nothing to normalize if the row is zero
+            if (rowsums[s_i] == 0)
+                continue;
+
+            for (size_t s_j = 0; s_j < n_state; ++s_j)
+                res[s_i + s_j * n_state] /= rowsums[s_i];
+        }
     }
 
     if (print)
@@ -1222,6 +1590,21 @@ inline std::vector< epiworld_double > DataBase<TSeq>::transition_probability(
                 nchar = l.length();
 
         std::string fmt = " - %-" + std::to_string(nchar) + "s";
+
+        std::string fmt_entry = " % 4.2f";
+        if (!normalize)
+        {
+            nchar = 0u;
+            for (auto & l: res)
+            {
+                std::string tmp = std::to_string(l);
+                if (tmp.length() > nchar)
+                    nchar = tmp.length();
+            }
+
+            fmt_entry = " % " + std::to_string(nchar) + ".0f";
+        } 
+
         
         printf_epiworld("\nTransition Probabilities:\n");
         for (size_t s_i = 0u; s_i < n_state; ++s_i)
@@ -1229,11 +1612,16 @@ inline std::vector< epiworld_double > DataBase<TSeq>::transition_probability(
             printf_epiworld(fmt.c_str(), states_labels[s_i].c_str());
             for (size_t s_j = 0u; s_j < n_state; ++s_j)
             {
-                if (std::isnan(res[s_i + s_j * n_state]))
+                if (
+                    std::isnan(res[s_i + s_j * n_state]) ||
+                    (res[s_i + s_j * n_state] < 1e-10)
+                )
                 {
                     printf_epiworld("     -");
                 } else {
-                    printf_epiworld(" % 4.2f", res[s_i + s_j * n_state]);
+                    printf_epiworld(
+                        fmt_entry.c_str(), res[s_i + s_j * n_state]
+                    );
                 }
             }
             printf_epiworld("\n");
@@ -1688,7 +2076,7 @@ inline bool DataBase<TSeq>::operator==(const DataBase<TSeq> & other) const
 }
 
 template<typename TSeq>
-inline void DataBase<TSeq>::generation_time(
+inline void DataBase<TSeq>::get_generation_time(
     std::vector< int > & agent_id,
     std::vector< int > & virus_id,
     std::vector< int > & time,
@@ -1740,7 +2128,7 @@ inline void DataBase<TSeq>::generation_time(
 }
 
 template<typename TSeq>
-inline void DataBase<TSeq>::generation_time(
+inline void DataBase<TSeq>::get_generation_time(
     std::string fn
 ) const
 {
@@ -1750,7 +2138,7 @@ inline void DataBase<TSeq>::generation_time(
     std::vector< int > time;
     std::vector< int > gentime;
 
-    generation_time(agent_id, virus_id, time, gentime);
+    get_generation_time(agent_id, virus_id, time, gentime);
 
     std::ofstream fn_file(fn, std::ios_base::out);
 
@@ -1758,7 +2146,7 @@ inline void DataBase<TSeq>::generation_time(
     if (!fn_file)
     {
         throw std::runtime_error(
-            "DataBase::generation_time: "
+            "DataBase::get_generation_time: "
             "Cannot open file " + fn + "."
         );
     }
@@ -1786,6 +2174,27 @@ inline void DataBase<TSeq>::generation_time(
 
 }
 
+template<typename TSeq>
+inline void DataBase<TSeq>::record_hospitalization(Agent<TSeq> & agent)
+{
+    m_hospitalizations.record(agent, *model);
+}
+
+template<typename TSeq>
+inline void DataBase<TSeq>::get_hospitalizations(
+    std::vector<int> & date,
+    std::vector<int> & virus_id,
+    std::vector<int> & tool_id,
+    std::vector<int> & count,
+    std::vector<double> & weight
+) const
+{
+    // Get the number of days from the model and add 1 since days are 0-indexed
+    // today() returns the last day, so we need today() + 1 for the full count
+    int ndays = model->today() + 1;
+    m_hospitalizations.get(ndays, date, virus_id, tool_id, count, weight);
+}
+
 #undef VECT_MATCH
 
 #endif
