@@ -1,6 +1,8 @@
 #ifndef EPIWORLD_VIRUS_MEAT_HPP
 #define EPIWORLD_VIRUS_MEAT_HPP
 
+#include "config.hpp"
+
 /**
  * @brief Factory function of VirusFun base on logit
  * 
@@ -13,8 +15,7 @@ template<typename TSeq>
 inline VirusFun<TSeq> virus_fun_logit(
     std::vector< int > vars,
     std::vector< double > coefs,
-    Model<TSeq> * model,
-    bool logit = true
+    Model<TSeq> * model
 ) {
 
     // Checking that there are features
@@ -56,10 +57,10 @@ inline VirusFun<TSeq> virus_fun_logit(
     for (auto c: coefs)
         coefs_f.push_back(static_cast<epiworld_double>(c));
 
-    VirusFun<TSeq> fun_infect = [coefs_f,vars,logit](
+    VirusFun<TSeq> fun_infect = [coefs_f,vars](
         Agent<TSeq> * agent,
-        Virus<TSeq> & virus,
-        Model<TSeq> * model
+        Virus<TSeq> &,
+        Model<TSeq> *
         ) -> epiworld_double {
 
         size_t K = coefs_f.size();
@@ -80,17 +81,191 @@ inline VirusFun<TSeq> virus_fun_logit(
 }
 
 template<typename TSeq>
-inline Virus<TSeq>::Virus(std::string name) {
+inline Virus<TSeq>::Virus()
+{
+
+    #ifdef EPI_DEBUG_VIRUS
+    counter_construct++;
+    #endif
+
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        baseline_sequence = -1;
+    }
+    else
+    {
+        baseline_sequence = nullptr;
+    }
+
+}
+
+template<typename TSeq>
+inline Virus<TSeq>::Virus(
+    std::string name
+    ) {
+
+    #ifdef EPI_DEBUG_VIRUS
+    counter_construct++;
+    #endif
+
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        baseline_sequence = -1;
+    }
+    else
+    {
+        baseline_sequence = nullptr;
+    }
+    
     set_name(name);
 }
 
+template<typename TSeq>
+inline Virus<TSeq>::Virus(
+    std::string name,
+    epiworld_double prevalence,
+    bool prevalence_as_proportion
+    ) {
+
+    #ifdef EPI_DEBUG_VIRUS
+    counter_construct++;
+    #endif
+
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        baseline_sequence = -1;
+    }
+    else
+    {
+        baseline_sequence = nullptr;
+    }
+
+    set_name(name);
+    set_distribution(
+        distribute_virus_randomly<TSeq>(
+            prevalence,
+            prevalence_as_proportion
+        )
+    );
+}
+
+#ifdef EPI_DEBUG_VIRUS
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_construct = 0;
+
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_copy_construct = 0;
+
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_move_construct = 0;
+
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_copy_assign = 0;
+
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_move_assign = 0;
+
+template<typename TSeq>
+std::atomic<int> Virus<TSeq>::counter_destruct = 0;
+
+template<typename TSeq>
+inline Virus<TSeq>::~Virus()
+{
+    counter_destruct++;
+}
+
+// Copy constructor
+template<typename TSeq>
+inline Virus<TSeq>::Virus(const Virus<TSeq>& other)
+    : agent(other.agent),
+      baseline_sequence(other.baseline_sequence),
+      virus_name(other.virus_name),
+      date(other.date),
+      id(other.id),
+      state_init(other.state_init),
+      state_post(other.state_post),
+      state_removed(other.state_removed),
+      queue_init(other.queue_init),
+      queue_post(other.queue_post),
+      queue_removed(other.queue_removed),
+      virus_functions(other.virus_functions)
+{
+    counter_copy_construct++;
+}
+
+// Move constructor
+template<typename TSeq>
+inline Virus<TSeq>::Virus(Virus<TSeq>&& other) noexcept
+    : agent(other.agent),
+      baseline_sequence(std::move(other.baseline_sequence)),
+      virus_name(std::move(other.virus_name)),
+      date(other.date),
+      id(other.id),
+      state_init(other.state_init),
+      state_post(other.state_post),
+      state_removed(other.state_removed),
+      queue_init(other.queue_init),
+      queue_post(other.queue_post),
+      queue_removed(other.queue_removed),
+      virus_functions(std::move(other.virus_functions))
+{
+    counter_move_construct++;
+    // other.agent = nullptr;
+}
+
+// Copy assignment
+template<typename TSeq>
+inline Virus<TSeq>& Virus<TSeq>::operator=(const Virus<TSeq>& other)
+{
+    if (this != &other) {
+        agent = other.agent;
+        baseline_sequence = other.baseline_sequence;
+        virus_name = other.virus_name;
+        date = other.date;
+        id = other.id;
+        state_init = other.state_init;
+        state_post = other.state_post;
+        state_removed = other.state_removed;
+        queue_init = other.queue_init;
+        queue_post = other.queue_post;
+        queue_removed = other.queue_removed;
+        virus_functions = other.virus_functions;
+        counter_copy_assign++;
+    }
+    return *this;
+}
+
+// Move assignment
+template<typename TSeq>
+inline Virus<TSeq>& Virus<TSeq>::operator=(Virus<TSeq>&& other) noexcept
+{
+    if (this != &other) {
+        agent = other.agent;
+        baseline_sequence = std::move(other.baseline_sequence);
+        virus_name = std::move(other.virus_name);
+        date = other.date;
+        id = other.id;
+        state_init = other.state_init;
+        state_post = other.state_post;
+        state_removed = other.state_removed;
+        queue_init = other.queue_init;
+        queue_post = other.queue_post;
+        queue_removed = other.queue_removed;
+        virus_functions = std::move(other.virus_functions);
+        other.agent = nullptr;
+        counter_move_assign++;
+    }
+    return *this;
+}
+#endif
+
 template<typename TSeq>
 inline void Virus<TSeq>::mutate(
     Model<TSeq> * model
 ) {
 
-    if (mutation_fun)
-        if (mutation_fun(agent, *this, model))
+    if (virus_functions->mutation)
+        if (virus_functions->mutation(agent, *this, model))
             model->get_db().record_virus(*this);
 
     return;
@@ -101,11 +276,11 @@ template<typename TSeq>
 inline void Virus<TSeq>::set_mutation(
     MutFun<TSeq> fun
 ) {
-    mutation_fun = MutFun<TSeq>(fun);
+    virus_functions->mutation = MutFun<TSeq>(fun);
 }
 
 template<typename TSeq>
-inline std::shared_ptr<TSeq> Virus<TSeq>::get_sequence()
+inline EPI_TYPENAME_TRAITS(TSeq, int) Virus<TSeq>::get_sequence()
 {
 
     return baseline_sequence;
@@ -121,6 +296,16 @@ inline void Virus<TSeq>::set_sequence(TSeq sequence)
 
 }
 
+template<>
+inline void Virus<int>::set_sequence(int sequence)
+{
+
+    baseline_sequence = sequence;
+    return;
+
+}
+
+
 template<typename TSeq>
 inline Agent<TSeq> * Virus<TSeq>::get_agent()
 {
@@ -175,8 +360,8 @@ inline epiworld_double Virus<TSeq>::get_prob_infecting(
 )
 {
 
-    if (probability_of_infecting_fun)
-        return probability_of_infecting_fun(agent, *this, model);
+    if (virus_functions->probability_of_infecting)
+        return virus_functions->probability_of_infecting(agent, *this, model);
         
     return EPI_DEFAULT_VIRUS_PROB_INFECTION;
 
@@ -190,8 +375,8 @@ inline epiworld_double Virus<TSeq>::get_prob_recovery(
 )
 {
 
-    if (probability_of_recovery_fun)
-        return probability_of_recovery_fun(agent, *this, model);
+    if (virus_functions->probability_of_recovery)
+        return virus_functions->probability_of_recovery(agent, *this, model);
         
     return EPI_DEFAULT_VIRUS_PROB_RECOVERY;
 
@@ -205,8 +390,8 @@ inline epiworld_double Virus<TSeq>::get_prob_death(
 )
 {
 
-    if (probability_of_death_fun)
-        return probability_of_death_fun(agent, *this, model);
+    if (virus_functions->probability_of_death)
+        return virus_functions->probability_of_death(agent, *this, model);
         
     return EPI_DEFAULT_VIRUS_PROB_DEATH;
 
@@ -218,8 +403,8 @@ inline epiworld_double Virus<TSeq>::get_incubation(
 )
 {
 
-    if (incubation_fun)
-        return incubation_fun(agent, *this, model);
+    if (virus_functions->incubation)
+        return virus_functions->incubation(agent, *this, model);
         
     return EPI_DEFAULT_INCUBATION_DAYS;
 
@@ -228,25 +413,25 @@ inline epiworld_double Virus<TSeq>::get_incubation(
 template<typename TSeq>
 inline void Virus<TSeq>::set_prob_infecting_fun(VirusFun<TSeq> fun)
 {
-    probability_of_infecting_fun = fun;
+    virus_functions->probability_of_infecting = fun;
 }
 
 template<typename TSeq>
 inline void Virus<TSeq>::set_prob_recovery_fun(VirusFun<TSeq> fun)
 {
-    probability_of_recovery_fun = fun;
+    virus_functions->probability_of_recovery = fun;
 }
 
 template<typename TSeq>
 inline void Virus<TSeq>::set_prob_death_fun(VirusFun<TSeq> fun)
 {
-    probability_of_death_fun = fun;
+    virus_functions->probability_of_death = fun;
 }
 
 template<typename TSeq>
 inline void Virus<TSeq>::set_incubation_fun(VirusFun<TSeq> fun)
 {
-    incubation_fun = fun;
+    virus_functions->incubation = fun;
 }
 
 template<typename TSeq>
@@ -258,7 +443,7 @@ inline void Virus<TSeq>::set_prob_infecting(const epiworld_double * prob)
             return *prob;
         };
     
-    probability_of_infecting_fun = tmpfun;
+    virus_functions->probability_of_infecting = tmpfun;
 }
 
 template<typename TSeq>
@@ -270,7 +455,7 @@ inline void Virus<TSeq>::set_prob_recovery(const epiworld_double * prob)
             return *prob;
         };
     
-    probability_of_recovery_fun = tmpfun;
+    virus_functions->probability_of_recovery = tmpfun;
 }
 
 template<typename TSeq>
@@ -282,7 +467,7 @@ inline void Virus<TSeq>::set_prob_death(const epiworld_double * prob)
             return *prob;
         };
     
-    probability_of_death_fun = tmpfun;
+    virus_functions->probability_of_death = tmpfun;
 }
 
 template<typename TSeq>
@@ -294,7 +479,7 @@ inline void Virus<TSeq>::set_incubation(const epiworld_double * prob)
             return *prob;
         };
     
-    incubation_fun = tmpfun;
+    virus_functions->incubation = tmpfun;
 }
 
 template<typename TSeq>
@@ -306,7 +491,7 @@ inline void Virus<TSeq>::set_prob_infecting(epiworld_double prob)
             return prob;
         };
     
-    probability_of_infecting_fun = tmpfun;
+    virus_functions->probability_of_infecting = tmpfun;
 }
 
 template<typename TSeq>
@@ -318,7 +503,7 @@ inline void Virus<TSeq>::set_prob_recovery(epiworld_double prob)
             return prob;
         };
     
-    probability_of_recovery_fun = tmpfun;
+    virus_functions->probability_of_recovery = tmpfun;
 }
 
 template<typename TSeq>
@@ -330,7 +515,7 @@ inline void Virus<TSeq>::set_prob_death(epiworld_double prob)
             return prob;
         };
     
-    probability_of_death_fun = tmpfun;
+    virus_functions->probability_of_death = tmpfun;
 }
 
 template<typename TSeq>
@@ -342,20 +527,20 @@ inline void Virus<TSeq>::set_incubation(epiworld_double prob)
             return prob;
         };
     
-    incubation_fun = tmpfun;
+    virus_functions->incubation = tmpfun;
 }
 
 template<typename TSeq>
 inline void Virus<TSeq>::set_post_recovery(PostRecoveryFun<TSeq> fun)
 {
-    if (post_recovery_fun)
+    if (virus_functions->post_recovery)
     {
         printf_epiworld(
             "Warning: a PostRecoveryFun is alreay in place (overwriting)."
             );
     }
 
-    post_recovery_fun = fun;
+    virus_functions->post_recovery = fun;
 }
 
 template<typename TSeq>
@@ -364,8 +549,8 @@ inline void Virus<TSeq>::post_recovery(
 )
 {
 
-    if (post_recovery_fun)
-        post_recovery_fun(agent, *this, model);    
+    if (virus_functions->post_recovery)
+        virus_functions->post_recovery(agent, *this, model);    
 
     return;
         
@@ -377,7 +562,7 @@ inline void Virus<TSeq>::set_post_immunity(
 )
 {
 
-    if (post_recovery_fun)
+    if (virus_functions->post_recovery)
     {
 
         std::string msg =
@@ -394,7 +579,7 @@ inline void Virus<TSeq>::set_post_immunity(
 
     // To make sure that we keep registering the virus
     ToolPtr<TSeq> __no_reinfect = std::make_shared<Tool<TSeq>>(
-        "Immunity (" + *virus_name + ")"
+        "Immunity (" + virus_name + ")"
     );
 
     __no_reinfect->set_susceptibility_reduction(prob);
@@ -412,13 +597,13 @@ inline void Virus<TSeq>::set_post_immunity(
             if (__no_reinfect->get_id() == -99)
                 m->get_db().record_tool(*__no_reinfect);
 
-            p->add_tool(__no_reinfect, m);
+            p->add_tool(*__no_reinfect, m);
 
             return;
 
         };
 
-    post_recovery_fun = tmpfun;
+    virus_functions->post_recovery = tmpfun;
 
 }
 
@@ -428,7 +613,7 @@ inline void Virus<TSeq>::set_post_immunity(
 )
 {
 
-    if (post_recovery_fun)
+    if (virus_functions->post_recovery)
     {
 
         std::string msg =
@@ -445,7 +630,7 @@ inline void Virus<TSeq>::set_post_immunity(
 
     // To make sure that we keep registering the virus
     ToolPtr<TSeq> __no_reinfect = std::make_shared<Tool<TSeq>>(
-        "Immunity (" + *virus_name + ")"
+        "Immunity (" + virus_name + ")"
     );
 
     __no_reinfect->set_susceptibility_reduction(prob);
@@ -461,13 +646,13 @@ inline void Virus<TSeq>::set_post_immunity(
             if (__no_reinfect->get_id() == -99)
                 m->get_db().record_tool(*__no_reinfect);
 
-            p->add_tool(__no_reinfect, m);
+            p->add_tool(*__no_reinfect, m);
 
             return;
 
         };
 
-    post_recovery_fun = tmpfun;
+    virus_functions->post_recovery = tmpfun;
 
 }
 
@@ -475,10 +660,7 @@ template<typename TSeq>
 inline void Virus<TSeq>::set_name(std::string name)
 {
 
-    if (name == "")
-        virus_name = nullptr;
-    else
-        virus_name = std::make_shared<std::string>(name);
+    virus_name = name;
 
 }
 
@@ -486,18 +668,10 @@ template<typename TSeq>
 inline std::string Virus<TSeq>::get_name() const
 {
 
-    if (virus_name)
-        return *virus_name;
-    
-    return "unknown virus";
+    return virus_name;
 
 }
 
-template<typename TSeq>
-inline std::vector< epiworld_double > & Virus<TSeq>::get_data() {
-    return data;
-}
-
 template<typename TSeq>
 inline void Virus<TSeq>::set_state(
     epiworld_fast_int init,
@@ -568,7 +742,6 @@ inline bool Virus<std::vector<int>>::operator==(
     ) const
 {
     
-
     EPI_DEBUG_FAIL_AT_TRUE(
         baseline_sequence->size() != other.baseline_sequence->size(),
         "Virus:: baseline_sequence don't match"
@@ -627,10 +800,20 @@ template<typename TSeq>
 inline bool Virus<TSeq>::operator==(const Virus<TSeq> & other) const
 {
     
-    EPI_DEBUG_FAIL_AT_TRUE(
-        *baseline_sequence != *other.baseline_sequence,
-        "Virus:: baseline_sequence don't match"
-    )
+    EPI_IF_TSEQ_LESS_EQ_INT( TSeq )
+    {
+        EPI_DEBUG_FAIL_AT_TRUE(
+            baseline_sequence != other.baseline_sequence,
+            "Virus:: baseline_sequence don't match"
+        )
+    }
+    else
+    {
+        EPI_DEBUG_FAIL_AT_TRUE(
+            baseline_sequence != other.baseline_sequence,
+            "Virus:: baseline_sequence don't match"
+        )
+    }
 
     EPI_DEBUG_FAIL_AT_TRUE(
         virus_name != other.virus_name,
@@ -675,7 +858,7 @@ template<typename TSeq>
 inline void Virus<TSeq>::print() const
 {
 
-    printf_epiworld("Virus         : %s\n", virus_name->c_str());
+    printf_epiworld("Virus         : %s\n", virus_name.c_str());
     printf_epiworld("Id            : %s\n", (id < 0)? std::string("(empty)").c_str() : std::to_string(id).c_str());
     printf_epiworld("state_init    : %i\n", static_cast<int>(state_init));
     printf_epiworld("state_post    : %i\n", static_cast<int>(state_post));
@@ -686,4 +869,23 @@ inline void Virus<TSeq>::print() const
 
 }
 
+template<typename TSeq>
+inline void Virus<TSeq>::distribute(Model<TSeq> * model)
+{
+
+    if (virus_functions->dist)
+    {
+
+        virus_functions->dist(*this, model);
+
+    }
+
+}
+
+template<typename TSeq>
+inline void Virus<TSeq>::set_distribution(VirusToAgentFun<TSeq> fun)
+{
+    virus_functions->dist = fun;
+}
+
 #endif
